{ This unit displays a form which is used to calculate the steady state
  solution to the model equations. The user must code the equations to calculate
  the steady state in the procedure calibrate. }
unit calculate;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  stypes, StdCtrls, ExtCtrls;

type
  TFmCalculate = class(TForm)
    MoCalibrate: TMemo;
    BtnYes: TButton;
    BtnNo: TButton;
    Bevel1: TBevel;
    GbStatus: TGroupBox;
    BtnOK: TButton;
    EdCalStatus: TEdit;
 
    procedure GetCalDrivers(Sender:TObject);
    Procedure calibrate;
    procedure BtnOKClick(Sender: TObject);
    procedure BtnYesClick(Sender: TObject);
 private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FmCalculate: TFmCalculate;

implementation

uses equations, frontend, fileio;

{$R *.DFM}

procedure TFmCalculate.BtnYesClick(Sender: TObject);
begin
  if caption = 'Calibrate' then
    EdCalStatus.text := 'Calibrating'
  else
    EdCalStatus.text := 'Calculating Steady State';
  GetCalDrivers(BtnYes);
  if caption = 'Calibrate' then
   begin
    calibrate;
    EdCalStatus.text := 'Calibration Complete';
   end
  else
   begin
//    CalibrateForm.calibrate(modeldef, t?, par, drive, stat, proc, false);
//    EdCalStatus.text := 'Steady State Calculation Complete';
   end;
  MainForm.MIChart.Enabled := false;
  MainForm.MITable.Enabled := false;
end;

procedure TFmCalculate.BtnOKClick(Sender: TObject);
begin
EdCalStatus.text:='';
//Close;
end;

procedure tFmCalculate.GetCalDrivers(Sender:TObject);
begin
  OpenDriverFile(driverfilename,ModelDef.numdrive,drive,flread);
  GetCurrentDrivers(caltime,drive);
  CloseDriverFile;
end;

procedure tFmCalculate.calibrate;
var
   q,BC,BN,BA,qmax,qmin,gC,gN,UCv,UNv,LCv,LNv,RCv,betaNe,RNe,LNe,Nfix,
   DC,DN,EN,kappaNm,epsilonC,epsilonN,THETA,qD,qL,w,sumLNv,sumNfix,
   alphaN,psi,q10psi,temp,LCD,LND,Nfixm,rNfixm:double;
   i,k,npar:integer;
   tempproc:processarray;

begin
   for i:= 1 to numspp do
      begin
         if par[NumSpPar*(i-1)+13].value<=0 then
                       par[NumSpPar*(i-1)+13].value:=0.001;
         if par[NumSpPar*(i-1)+15].value<=0 then
                       par[NumSpPar*(i-1)+15].value:=0.001;
      end ;
   processes(Caltime,stat,tempproc,false);
   k:=ModelDef.numstate;
   qL:=0;
   sumLNv:=0;
   sumNfix:=0;
   for i:= 1 to numspp do
      begin 
         BC:=stat[3*(i-1)+1].value;
         BN:=stat[3*(i-1)+2].value; 
         qmax:= par[NumSpPar*(i-1)+8].value;  
         BA :=tempproc[NumSpProc*(i-1)+k+1 ].value;
         if BN>0 then q:=BC/BN else q:=1;
         if BC>0 then qmin:=BA*qmax*q/(BC*qmax-(BC-BA)*q) else qmin:= 1; 
         par[NumSpPar*(i-1)+7 ].value:=qmin;
      end;    
   processes(Caltime,stat,tempproc,false);
   for i:= 1 to numspp do
      begin
         BC:=stat[3*(i-1)+1].value;
         BN:=stat[3*(i-1)+2].value;
         qmax:= par[NumSpPar*(i-1)+8].value;
         gC:= par[NumSpPar*(i-1)+13].value;
         gN:= par[NumSpPar*(i-1)+15].value;
         if gC<=0 then gc:=0.001;
         if gN<=0 then gn:=0.001;
         BA :=tempproc[NumSpProc*(i-1)+k+1 ].value;
         UCv:=tempproc[NumSpProc*(i-1)+k+8 ].value;
         UNv:=tempproc[NumSpProc*(i-1)+k+10].value;
         LCv:=tempproc[NumSpProc*(i-1)+k+11].value;
         LNv:=tempproc[NumSpProc*(i-1)+k+12].value;
         qL:=qL+LCv;
         sumLNv:=sumLNv+LNv;
         RCv:=tempproc[NumSpProc*(i-1)+k+13].value; 
         Nfix:=tempproc[NumSpProc*(i-1)+k+15].value;
         sumNfix:=sumNfix+Nfix;
         if BN>0 then q:=BC/BN else q:=1;
         if BC>0 then qmin:=BA*qmax*q/(BC*qmax-(BC-BA)*q) else qmin:= 1;
         if UCv>0 then gC:=gC*(LCv+RCv)/UCv else gC:=0;
         if UNv>0 then gN:=gN*(LNv-Nfix)/UNv else gN:=0;
         par[NumSpPar*(i-1)+7 ].value:=qmin;
         par[NumSpPar*(i-1)+13].value:=gC;
         par[NumSpPar*(i-1)+15].value:=gN;
      end;
   DC      :=stat[3*numspp+1].value;
   DN      :=stat[3*numspp+2].value;
   EN      :=stat[3*numspp+3].value;
   temp    :=drive[2].value;
   RNe     :=drive[4].value;
   npar    :=ParCount(k+NumSpProc*numspp+1);
   betaNe  :=par[npar+10 ].value;
   kappaNm :=par[npar+15].value;
   epsilonC:=par[npar+16].value;
   epsilonN:=par[npar+17].value;
   Q10psi  :=par[npar+19].value;
   rNfixm  :=par[npar+24].value;
   LNe     :=tempproc[k+NumSpProc*numspp+9].value;
   THETA   :=tempproc[k+NumSpProc*numspp+11].value;
   LCD     :=tempproc[k+NumSpProc*numspp+18].value;
   LND     :=tempproc[k+NumSpProc*numspp+19].value;
   Nfixm   :=tempproc[k+NumSpProc*numspp+22].value;

   betaNe  :=betaNe*(RNe+sumNfix-LND+Nfixm)/LNe;
   psi:=epsilonC*THETA/epsilonN;
   psi:=(qL-LCD-Nfixm*rNfixm-psi*(sumLNv-LND+Nfixm))/(DC-psi*DN);
   alphaN:=(psi*DN+LND-SumLNv-Nfixm);
   alphaN:=alphaN*psi*DC/(epsilonN*psi*DC-alphaN*THETA);
   alphaN:=(alphaN-psi*DN)*(THETA*DN*(kappaNm+EN))/(psi*sqr(DC)*EN);

   par[npar+10].value:=betaNe;
   par[npar+14].value:=alphaN;
   par[npar+18].value:=psi/exp(ln(Q10psi)*temp/10);
end;

end.
